package motiflab.plugins.gibbssamplerdemo;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;
import javax.swing.border.BevelBorder;
import motiflab.engine.DataListener;
import motiflab.engine.ExecutionError;
import motiflab.engine.MotifLabClient;
import motiflab.engine.MotifLabEngine;
import motiflab.engine.Plugin;
import motiflab.engine.SystemError;
import motiflab.engine.data.DNASequenceData;
import motiflab.engine.data.DNASequenceDataset;
import motiflab.engine.data.Data;
import motiflab.engine.data.Motif;
import motiflab.engine.data.NumericDataset;
import motiflab.engine.data.NumericSequenceData;
import motiflab.engine.data.OutputData;
import motiflab.engine.data.Region;
import motiflab.engine.data.RegionDataset;
import motiflab.engine.data.RegionSequenceData;
import motiflab.engine.data.Sequence;
import motiflab.engine.data.SequenceCollection;
import motiflab.engine.protocol.StandardProtocol;
import motiflab.gui.MotifLabGUI;
import motiflab.gui.MotifLogo;
import motiflab.gui.VisualizationSettings;


/**
 * An animated demonstration (on user selected data) of the "Gibbs Sampling" motif discovery method described in the paper
 * CE Lawrence, SF Altschul et al. (1993) "Detecting Subtle Sequence Signals: A Gibbs Sampling Strategy for Multiple Alignment" Science 262(5131) : 208-214
 * 
 * @author kjetikl
 */
public class GibbsSamplerDemo implements Plugin, DataListener {
    
    private String GIBBS_SAMPLING_REGIONS_DATASETNAME="Gibbs_sampling_regions";
    private String GIBBS_SAMPLING_DISTRIBUTION_DATASETNAME="Gibbs_sampling_distribution";
    private String GIBBS_SAMPLING_MOTIF_DATASETNAME="Gibbs_sampling_motif";
    
    private MotifLabGUI gui;
    private MotifLabEngine engine;
    private JDialog dialog=null;
    private JSpinner motifSizeSpinner;
    private JSpinner samplingStepsSpinner;   
    private JButton startButton;  
    private JButton stopButton;
    private JButton stepButton;
    private JButton resetButton;      
    private JButton closeButton;    
    private JButton documentationButton;    
    private JButton protocolButton;        
    private JButton alignButton;
    private JProgressBar progressBar;
    private JComboBox DNAtrackComboBox;
    private MotifLogoPanel motiflogopanel;
    private GraphPanel graphPanel;
    private AnimationWorker animationWorker=new AnimationWorker();
    private AnimateAlign alignAnimator=null; 
    
    private boolean selectSequencesAtRandom=false; // if false, sequences to hold out will be selected on a round-robin basis    
    
    private int stepCounter=0;
    private int maxSteps=1000;
    private int motifsize=0;
    private RegionDataset regiondataset=null;
    private NumericDataset numerictrack=null; // sampling distribution
    private DNASequenceDataset dnatrack=null;
    private Motif currentmotif=null;          
    private double[] backgroundDistribution=new double[4];
        
    private ArrayList<Sequence> sequences;
    
    
    private void showDialog() {                           
        setupDialog();
        dnatrack=null;
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=gui.getGUIFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));
        dialog.setVisible(true); 
        engine.addDataListener(this);
    }
    
    private void setupDialog() {
       dialog=new JDialog(gui.getGUIFrame(),"Gibbs Sampler Demo",false); 
       dialog.setLayout(new BorderLayout());
       JPanel bottomPanel=new JPanel(new BorderLayout());
       JPanel bottomPanelRight=new JPanel(new FlowLayout(FlowLayout.RIGHT));
       JPanel bottomPanelLeft=new JPanel(new FlowLayout(FlowLayout.LEFT));
       bottomPanel.add(bottomPanelLeft,BorderLayout.WEST);
       bottomPanel.add(bottomPanelRight,BorderLayout.EAST);  
       dialog.add(bottomPanel,BorderLayout.SOUTH);  
       progressBar=new JProgressBar(0, 100);
       progressBar.setPreferredSize(new Dimension(300,25));
       progressBar.setVisible(false);
       bottomPanelLeft.add(progressBar);       
       dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
       closeButton=new JButton("Close");
       closeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                stopAnimation();
                resetAnimation(false);
                engine.removeDataListener(GibbsSamplerDemo.this);
                dialog.setVisible(false);
                dialog.dispose();
                dialog=null;       
            }
        });
       bottomPanelRight.add(closeButton);
       JPanel controlsPanel1=new JPanel(new BorderLayout());
       controlsPanel1.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
       dialog.add(controlsPanel1,BorderLayout.CENTER);    
       JPanel controlsPanel2=new JPanel(new BorderLayout());
       controlsPanel1.add(controlsPanel2,BorderLayout.CENTER);            
       JPanel sizenandstepsPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));       
       JPanel startstopstepPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));   
       JPanel documentationControls=new JPanel(new FlowLayout(FlowLayout.RIGHT));        
       JPanel bottomControlsPanel=new JPanel(new BorderLayout());         
       JPanel logopanel=new JPanel(new BorderLayout()); // this is a wrapper for the inner motiflogopanel
       logopanel.setPreferredSize(new Dimension(450, 150));
       logopanel.setMinimumSize(new Dimension(100, 30));
       logopanel.setBorder(BorderFactory.createCompoundBorder(
               BorderFactory.createEmptyBorder(8,8,2,8),    // we use the splitpane divier as border instead
               BorderFactory.createEtchedBorder()
       ));
       sizenandstepsPanel.setBorder(BorderFactory.createEmptyBorder(0, 8, 0, 8));       
       bottomControlsPanel.setBorder(BorderFactory.createEmptyBorder(0, 8, 0, 8));         
       motiflogopanel=new MotifLogoPanel();
       logopanel.add(motiflogopanel);      
       graphPanel = new GraphPanel();
       graphPanel.setBorder(BorderFactory.createCompoundBorder(
               BorderFactory.createEmptyBorder(2,8,8,8), // we use the splitpane divier as border instead
               BorderFactory.createBevelBorder(BevelBorder.LOWERED)
       ));    
       graphPanel.setPreferredSize(new Dimension(450, 50));
       graphPanel.setSize(new Dimension(450, 50));
       JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, logopanel, graphPanel); 
       controlsPanel1.add(sizenandstepsPanel,BorderLayout.NORTH);  
       controlsPanel2.add(splitPane,BorderLayout.CENTER);  
       controlsPanel2.add(bottomControlsPanel,BorderLayout.SOUTH);  
       bottomControlsPanel.add(startstopstepPanel,BorderLayout.CENTER);
       bottomControlsPanel.add(documentationControls,BorderLayout.EAST);
       ArrayList<String> tracks=engine.getNamesForAllDataItemsOfType(DNASequenceDataset.class);

       String[] tracksArray=new String[tracks.size()];
       tracksArray=tracks.toArray(tracksArray);
       DefaultComboBoxModel tracksModel=new DefaultComboBoxModel(tracksArray);      
       DNAtrackComboBox = new JComboBox(tracksModel);
       motifSizeSpinner=new JSpinner(new SpinnerNumberModel(8, 4, 30, 1));
       samplingStepsSpinner=new JSpinner(new SpinnerNumberModel(200, 1, 2000, 1));  
       sizenandstepsPanel.add(new JLabel("DNA track  "));
       sizenandstepsPanel.add(DNAtrackComboBox);       
       sizenandstepsPanel.add(new JLabel("    Motif size  "));
       sizenandstepsPanel.add(motifSizeSpinner);         
       sizenandstepsPanel.add(new JLabel("    Sampling steps  "));       
       sizenandstepsPanel.add(samplingStepsSpinner);            
       
       ImageIcon startIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("execute16.png"));       
       startButton=new JButton(startIcon);
       startButton.setToolTipText("Start");        
       startButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                startAnimation();
            }
       });      
       ImageIcon stopIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("stop16.png"));           
       stopButton=new JButton(stopIcon);
       stopButton.setToolTipText("Stop");            
       stopButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                stopAnimation();
            }
       });              
       ImageIcon stepIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("step16.png"));           
       stepButton=new JButton(stepIcon);
       stepButton.setToolTipText("Step");            
       stepButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                stepAnimation();
            }
       });
       ImageIcon resetIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("reset16.png"));              
       resetButton=new JButton(resetIcon);
       resetButton.setToolTipText("Reset");            
       resetButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                resetAnimation(true);
            }
       }); 
       ImageIcon documentationIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("help_icon.png"));
       documentationButton=new JButton(documentationIcon);
       documentationButton.setToolTipText("Show documentation");       
       documentationButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showDocumentation();
            }
       }); 
       ImageIcon protocolIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("small_document_icon.png"));
       protocolButton=new JButton(protocolIcon);
       protocolButton.setToolTipText("Open example protocol");
       protocolButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showProtocol();
            }
       });    
       ImageIcon alignIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("align16.png"));        
       alignButton=new JButton(alignIcon);
       alignButton.setToolTipText("Align sites");
       alignButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                alignSites();
            }
       });      
       documentationControls.add(alignButton);        
       documentationControls.add(protocolButton);         
       documentationControls.add(documentationButton);        
       startstopstepPanel.add(startButton); 
       startstopstepPanel.add(stopButton);         
       startstopstepPanel.add(stepButton);        
       startstopstepPanel.add(resetButton);    
       startButton.setEnabled(true);
       stepButton.setEnabled(true); 
       stopButton.setEnabled(false);  
       resetButton.setEnabled(false);         
       dialog.pack();
       if (tracks.isEmpty()) {startButton.setEnabled(false);stepButton.setEnabled(false);} // if no DNA track is available, disable start/step buttons
    }
    
    
    
    private void startAnimation() {
        maxSteps=(Integer)samplingStepsSpinner.getValue();
        progressBar.setMaximum(maxSteps);
        if (stepCounter==0) progressBar.setString("0/"+maxSteps);
        progressBar.setVisible(true);
        startButton.setEnabled(false);        
        stopButton.setEnabled(true);
        stepButton.setEnabled(false);        
        resetButton.setEnabled(false); 
        motifSizeSpinner.setEnabled(false);
        samplingStepsSpinner.setEnabled(false);        
        DNAtrackComboBox.setEnabled(false);
        animationWorker=new AnimationWorker();
        animationWorker.execute();
    }
    
    private void stopAnimation() {
        progressBar.setVisible(true);
        maxSteps=(Integer)samplingStepsSpinner.getValue();        
        resetButton.setEnabled(true);        
        stopButton.setEnabled(false);
        boolean hasDNATracks=(DNAtrackComboBox.getModel().getSize()>0);
        stepButton.setEnabled(hasDNATracks);        
        startButton.setEnabled(hasDNATracks);       
        if (animationWorker!=null) {
            animationWorker.stopAnimating();
            animationWorker=null;
        }
    }  

    private void stepAnimation() {
       maxSteps=(Integer)samplingStepsSpinner.getValue();  
       progressBar.setMaximum(maxSteps);
       progressBar.setVisible(true);         
       resetButton.setEnabled(true);
       motifSizeSpinner.setEnabled(false);
       samplingStepsSpinner.setEnabled(false);         
       DNAtrackComboBox.setEnabled(false);       
       performOneAnimationStepOnEDT();
    }      
    
    private void resetAnimation(boolean clearData) {
        progressBar.setVisible(false);         
        resetButton.setEnabled(false); 
        stopButton.setEnabled(false);
        boolean hasDNATracks=(DNAtrackComboBox.getModel().getSize()>0);
        stepButton.setEnabled(hasDNATracks);        
        startButton.setEnabled(hasDNATracks);        
        motifSizeSpinner.setEnabled(true);  
        samplingStepsSpinner.setEnabled(true);          
        DNAtrackComboBox.setEnabled(DNAtrackComboBox.getModel().getSize()>0);        
        reset(clearData);
    }
    
    private void endAnimation() {
        progressBar.setVisible(false);
        startButton.setEnabled(false);        
        stopButton.setEnabled(false);
        stepButton.setEnabled(false);        
        resetButton.setEnabled(true);      
        stepCounter=0;
    }
    
    private void resetData() { // clears the current data in order to start over fresh (only values)
        if (regiondataset!=null) {
            regiondataset=(RegionDataset)regiondataset.clone();
            regiondataset.clearRegions();
        }
        if (numerictrack!=null) {
            numerictrack=(NumericDataset)numerictrack.clone();   
            numerictrack.setAllPositionsToValue(0);              
        }
        if (currentmotif!=null) {
            currentmotif=(Motif)currentmotif.clone();           
            currentmotif.setConsensusMotifAndUpdatePWM("N"); // this will not show up anyway
        }
        updateData();
        graphPanel.reset();
        if (currentmotif!=null) motiflogopanel.setMotif(currentmotif);
    }
    
    private void performOneAnimationStepOnEDT() { 
        if (regiondataset!=null) { // create local clones of the ones already existing in the engine instead of updating directly into existing objects
            regiondataset=(RegionDataset)regiondataset.clone();
            numerictrack=(NumericDataset)numerictrack.clone();
            currentmotif=(Motif)currentmotif.clone();
        } 
        step(); // update local dataobjects         
        updateData(); // commit new local copies to engine 
        motiflogopanel.setMotif(currentmotif);
        double IC=currentmotif.getICcontent()/currentmotif.getLength(); // normalize by motifLength to reach a value between 0 and 2
        graphPanel.addStep(IC);
        progressBar.setValue(stepCounter);
        progressBar.setStringPainted(true);
        progressBar.setString(stepCounter+"/"+maxSteps);
        if (stepCounter>=maxSteps) endAnimation();
        dialog.repaint();
        gui.redraw();
    }
    
    private void showProtocol() {
        try {
            
            InputStream stream=this.getClass().getResourceAsStream("/motiflab/plugins/gibbssamplerdemo/example_protocol.txt");
            StandardProtocol exampleProtocol=new StandardProtocol(engine, stream);
            String protocolName=gui.getProtocolEditor().getProtocolManager().getDefaultProtocolName();
            exampleProtocol.setName(protocolName);
            gui.getProtocolEditor().newProtocol(exampleProtocol);
            gui.getMainWindow().showProtocolEditor(); 
        } catch (Exception e) {
            JOptionPane.showMessageDialog(dialog, "Unable to open example protocol: "+e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }                     
    }
    
    private void showDocumentation() {
        if (engine.dataExists("GibbsSampler", OutputData.class)) {
            gui.executeInternalCommand("show=GibbsSampler");
        } else {
            try {
                InputStream stream=this.getClass().getResourceAsStream("/motiflab/plugins/gibbssamplerdemo/GibbsSampler_documentation.html");
                ArrayList<String> lines=engine.readStreamContents(stream);
                OutputData output=new OutputData("GibbsSampler");
                output.append(lines, "HTML");
                output.setShowAsHTML(true);                
                engine.storeDataItem(output);
                gui.executeInternalCommand("show=GibbsSampler");                
            } catch (Exception e) {
                JOptionPane.showMessageDialog(dialog, "Unable to open documentation: "+e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            }                     
        }      
    }   
    
    private void alignSites() {
        if (alignAnimator!=null || dnatrack==null || regiondataset==null) return; // do not start twice
        alignButton.setEnabled(false);
        alignAnimator=new AnimateAlign();
        alignAnimator.execute();
    }
    
    
    private class AnimateAlign extends SwingWorker {   
        private ArrayList<String> sequencesToProcess;
        private HashMap<String,int[]> viewportInfo;
        private VisualizationSettings vizSettings;
        private double steps=20;
        private int i=0;
        
        @Override
        protected Object doInBackground() throws Exception {
            sequencesToProcess=(ArrayList<String>)engine.getNamesForAllDataItemsOfType(Sequence.class);
            viewportInfo=new HashMap<>();
            RegionDataset regionDataset=(RegionDataset)engine.getDataItem(GIBBS_SAMPLING_REGIONS_DATASETNAME, RegionDataset.class);
            if (regionDataset==null) return null;
            vizSettings=gui.getVisualizationSettings();
            for (String sequenceName:sequencesToProcess) { // find all middle points
                RegionSequenceData data=(RegionSequenceData)regionDataset.getSequenceByName(sequenceName);
                Region region=(data!=null)?data.getFirstRegion():null;
                if (region!=null) {
                    int start=region.getGenomicStart();
                    int halfsize=(int)(region.getLength()/2);
                    int middle=start+halfsize;
                    int currentVPstart=vizSettings.getSequenceViewPortStart(sequenceName);
                    int currentVPend=vizSettings.getSequenceViewPortEnd(sequenceName);                   
                    viewportInfo.put(sequenceName, new int[]{middle,halfsize,currentVPstart,currentVPend});
                    vizSettings.setSequenceConstrained(sequenceName, false);

                } 
            }
            for (i=0;i<=steps;i++) {             
                Runnable runner=new Runnable() {
                    @Override
                    public void run() {
                        for (String sequenceName:sequencesToProcess) { // find all middle points
                            int[] info=viewportInfo.get(sequenceName);
                            if (info==null) continue;
                            int middle=info[0];
                            int halfsize=info[1];
                            int VpS=info[2];
                            int VpE=info[3];   
                            int VPsize=VpE-VpS;
                            int VpSnew=middle-VPsize/2+halfsize;
                            int distance=VpSnew-VpS;
                            int newpos=VpS+(int)(distance*((double)i/(double)steps));
                            int VpEnew=newpos+VPsize;
                            vizSettings.setSequenceViewPort(sequenceName, newpos, VpEnew);
                        }                          
                        vizSettings.redraw();
                    }
                };
                SwingUtilities.invokeAndWait(runner);    
                // Thread.sleep(25);
            }
            return null;
        }

        @Override
        protected void done() {
            alignButton.setEnabled(true);
            alignAnimator=null;
        }            
    }    
    
    
    
    private class AnimationWorker extends SwingWorker {   
        private boolean stop=false;
        
        public void stopAnimating() {
            stop=true;
        }
        @Override
        protected Object doInBackground() throws Exception {
            while (stepCounter<maxSteps) {
                if (stop) throw new InterruptedException();
                if (regiondataset!=null) { // create local clones of the ones already existing in the engine instead of updating directly into existing objects
                    regiondataset=(RegionDataset)regiondataset.clone();
                    numerictrack=(NumericDataset)numerictrack.clone();
                    currentmotif=(Motif)currentmotif.clone();
                }
                step(); // update local dataobjects    
                Runnable runner=new Runnable() {
                    @Override
                    public void run() {                        
                        updateData(); // commit new local copies to engine      
                        motiflogopanel.setMotif(currentmotif);
                        double IC=currentmotif.getICcontent()/currentmotif.getLength(); // normalize by motifLength to reach a value between 0 and 2
                        graphPanel.addStep(IC);                        
                        progressBar.setValue(stepCounter);
                        progressBar.setStringPainted(true);
                        progressBar.setString(stepCounter+"/"+maxSteps);
                        dialog.repaint();
                        gui.redraw();
                    }
                };
                SwingUtilities.invokeAndWait(runner);    
                Thread.sleep(100);
            }
            Runnable runner=new Runnable() {
                @Override
                public void run() {                        
                     endAnimation();
                }
            };
            SwingUtilities.invokeAndWait(runner);
            return null;
        }

        @Override
        protected void done() {
            progressBar.setValue(stepCounter);
            progressBar.setStringPainted(true);
            progressBar.setString(stepCounter+"/"+maxSteps);
        }            
    }
    
    
    
    
    private class MotifLogoPanel extends JPanel {
        private MotifLogo motiflogo;  
        private JCheckBoxMenuItem scaleByICMenu;
        private JCheckBoxMenuItem sortByFreqMenu;
        private JPopupMenu contextmenu;
        
        public MotifLogoPanel() {
            super();  
            motiflogo=new MotifLogo(gui.getVisualizationSettings().getBaseColors());
            motiflogo.setDrawBorder(false);
            motiflogo.setScaleByIC(true);
            motiflogo.setSortByFrequency(true);
            contextmenu=new JPopupMenu();
            scaleByICMenu=new JCheckBoxMenuItem("Scale by IC",motiflogo.getScaleByIC());
            sortByFreqMenu=new JCheckBoxMenuItem("Sort by frequency",motiflogo.getSortByFrequency());
            ActionListener listener=new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem source=(JCheckBoxMenuItem)e.getSource();
                    if (source==scaleByICMenu) {
                        motiflogo.setScaleByIC(scaleByICMenu.isSelected());
                    } else if (source==sortByFreqMenu) {
                        motiflogo.setSortByFrequency(sortByFreqMenu.isSelected());
                    }
                    motiflogopanel.repaint();
                }
            };
            scaleByICMenu.addActionListener(listener);
            sortByFreqMenu.addActionListener(listener);
            contextmenu.add(scaleByICMenu);
            contextmenu.add(sortByFreqMenu);
            this.addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    maybeShowPopup(e);
                }
                public void mouseReleased(MouseEvent e) {
                    maybeShowPopup(e);
                }
                private void maybeShowPopup(MouseEvent e) {
                    if (e.isPopupTrigger()) {contextmenu.show(e.getComponent(),e.getX(), e.getY());}
                }
            });           
        }
        
        public void setMotif(Motif motif) {
            motiflogo.setMotif(motif);
        }
        
        @Override
        public void paintComponent(Graphics g) {            
            int width=this.getWidth();
            int height=this.getHeight();
            int margin=0;
            g.setColor(Color.WHITE);
            g.drawRect(margin, margin, width-(margin+margin), height-(margin+margin));
            motiflogo.paintLogo(g, margin, margin, width-(margin+margin), height-(margin+margin), false, false,false);
        }   
    }
    
    private class GraphPanel extends JPanel {
        
        private Stroke DASHED_STROKE=new BasicStroke(1f,BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1f, new float[]{2f,3f}, 0f);
        private Stroke LINE_STROKE=new BasicStroke(2f); 
        
        ArrayList<Double> graphvalues;
        Color veryLightGray=new Color(225,225,225);
        
        public GraphPanel() {
            super();   
            graphvalues=new ArrayList<>();
        }

        public void reset() {
            graphvalues.clear();
            repaint();
        }
        
        public void addStep(double value) {
            graphvalues.add(value);
        }
        
        @Override
        public void paintComponent(Graphics g) {      
            Insets insets=getBorder().getBorderInsets(this);
            int width=this.getWidth();   // this does not accound for borders
            int height=this.getHeight(); // this does not accound for borders
            int graphMargin=3; // top and bottom border for graph
            int useHeight=height-(graphMargin+graphMargin+insets.top+insets.bottom);
            int useWidth=width-(insets.left+insets.right);
            Stroke oldstroke=((Graphics2D)g).getStroke();
            g.setColor(Color.WHITE);
            g.fillRect(insets.left,insets.top,width-(insets.left+insets.right),height-(insets.top+insets.bottom));
            ((Graphics2D)g).setStroke(DASHED_STROKE);            
            g.setColor(veryLightGray);
            int graphTop=graphMargin+insets.top;
            g.drawLine(insets.left, graphTop+(int)Math.round(useHeight*0.00), insets.left+useWidth, graphTop+(int)Math.round(useHeight*0.00));              
            g.drawLine(insets.left, graphTop+(int)Math.round(useHeight*0.25), insets.left+useWidth, graphTop+(int)Math.round(useHeight*0.25));            
            g.drawLine(insets.left, graphTop+(int)Math.round(useHeight*0.50), insets.left+useWidth, graphTop+(int)Math.round(useHeight*0.50));
            g.drawLine(insets.left, graphTop+(int)Math.round(useHeight*0.75), insets.left+useWidth, graphTop+(int)Math.round(useHeight*0.75)); 
            g.drawLine(insets.left, graphTop+(int)Math.round(useHeight*1.00), insets.left+useWidth, graphTop+(int)Math.round(useHeight*1.00));             
            double scaleX=(double)useWidth/(double)maxSteps;
            double scaleY=useHeight/2.0; // since 2 is largest IC-value
            Object hint=((Graphics2D)g).getRenderingHint(RenderingHints.KEY_ANTIALIASING);
            ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            int size=graphvalues.size();
            if (size==0) {}
            else if (size==1) {
                int xValue=insets.left;
                int yValue=graphTop+useHeight-(int)Math.round(graphvalues.get(0)*scaleY);                 
                g.setColor(Color.BLUE);            
                ((Graphics2D)g).setStroke(LINE_STROKE);             
                g.drawLine(xValue,yValue,xValue,yValue);             
            } else {
                int[] xPoints=new int[maxSteps];
                int[] yPoints=new int[maxSteps];
                for (int i=0;i<graphvalues.size();i++) {
                    xPoints[i]=insets.left+(int)Math.round(i*scaleX);
                    yPoints[i]=graphTop+useHeight-(int)Math.round(graphvalues.get(i)*scaleY);               
                }
                g.setColor(Color.BLUE);            
                ((Graphics2D)g).setStroke(LINE_STROKE);             
                g.drawPolyline(xPoints, yPoints, graphvalues.size());
            }
            ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, hint);    
            ((Graphics2D)g).setStroke(oldstroke);
        }   
    }    
    
    
    // ----- Listen to changes in data --------
    
    @Override
    public void dataAdded(Data data) {
        if (data instanceof DNASequenceDataset && dialog!=null) {
            DefaultComboBoxModel model=(DefaultComboBoxModel)DNAtrackComboBox.getModel();
            int oldsize=model.getSize();
            model.addElement(data.getName());
            if (oldsize==0) {
                startButton.setEnabled(true);
                stepButton.setEnabled(true); // enable buttons when adding first track
            }
            dialog.pack();
        } else if (data instanceof Sequence && (sequences!=null || regiondataset!=null || dnatrack!=null)) { // sequences are added and we already have data
            if (stepCounter>0) {
                stopAnimation(); // just in case
                startButton.setEnabled(false);
                stopButton.setEnabled(false);
                stepButton.setEnabled(false);
                resetButton.setEnabled(true);
            }               
            dnatrack=null;
            sequences=null;
            regiondataset=null;
            initializeDatasets(); // rebuild the datasets from scratch
            updateData(); // commit new local copies to engine
        }
    }

    @Override
    public void dataRemoved(Data data) {
        if (data instanceof DNASequenceDataset && dialog!=null) {
            DefaultComboBoxModel model=(DefaultComboBoxModel)DNAtrackComboBox.getModel();
            int oldsize=model.getSize();
            model.removeElement(data.getName());            
            if (oldsize==1 && model.getSize()==0) { // last track has been removed
                startButton.setEnabled(false);
                stepButton.setEnabled(false); 
            }                  
            if (data.getName().equals(dnatrack.getName())) { // removed the DNA dataset we are searching in => clear all
                dnatrack=null;
                sequences=null;
                regiondataset=null;
                if (stepCounter>0) {
                    stopAnimation(); // just in case
                    startButton.setEnabled(false);
                    stopButton.setEnabled(false);
                    stepButton.setEnabled(false);
                    resetButton.setEnabled(true);
                }               
            } 
            dialog.pack();
        } else if (data instanceof Sequence && (sequences!=null || regiondataset!=null  || dnatrack!=null)) { // sequences are removed and we already have data
            if (stepCounter>0) {
                stopAnimation(); // just in case
                startButton.setEnabled(false);
                stopButton.setEnabled(false);
                stepButton.setEnabled(false);
                resetButton.setEnabled(true);
            }           
            dnatrack=null;
            sequences=null;
            regiondataset=null;            
            initializeDatasets(); // rebuild the datasets from scratch
            updateData(); // commit new local copies to engine
        }
    }      
    
    @Override
    public void dataUpdate(Data oldvalue, Data newvalue) {
        // check if a DNA dataset has changed type
        if (oldvalue instanceof DNASequenceDataset && !(newvalue instanceof DNASequenceDataset)) { // data is no longer a DNA track
            dataRemoved(oldvalue); // this track is no longer a DNA track
        } else if (newvalue instanceof DNASequenceDataset && !(oldvalue instanceof DNASequenceDataset)) { // data has changed from something else into a DNA track
            dataAdded(newvalue); // this track is no longer a DNA track
        } else if ((oldvalue instanceof Sequence || newvalue instanceof Sequence)) {
            if (stepCounter>0) {
                stopAnimation(); // just in case
                startButton.setEnabled(false);
                stopButton.setEnabled(false);
                stepButton.setEnabled(false);
                resetButton.setEnabled(true);
            }               
            dnatrack=null;
            sequences=null;   
            regiondataset=null;            
            initializeDatasets(); // rebuild the datasets
            updateData(); // commit new local copies to engine
        }
    }   
    
    
    @Override
    public void dataAddedToSet(Data parentDataset, Data child) { }

    @Override
    public void dataRemovedFromSet(Data parentDataset, Data child) { }

    @Override
    public void dataUpdated(Data data) { }    
    
    
    
    // ----- Gibbs sampling process -----
    
    private void setupSequences() {
        SequenceCollection allSequences=engine.getDefaultSequenceCollection();
        sequences=allSequences.getAllSequences(engine);    
        if (dnatrack==null) {
           stepCounter=0;
           graphPanel.reset();
           String dnatrackname=(String)DNAtrackComboBox.getSelectedItem();
           dnatrack=(DNASequenceDataset)engine.getDataItem(dnatrackname, DNASequenceDataset.class);
           if (dnatrack==null) return; // just in case it has disappeared
       }        
    }
    
    
    private void reset(boolean clearData) {
       stepCounter=0;      
       if (clearData) resetData();
    }    
    
    private void step() {
       if (sequences==null || sequences.isEmpty() || dnatrack==null) setupSequences();  
       if (sequences==null || sequences.isEmpty() || dnatrack==null) return;

       boolean firstStep=(stepCounter==0);
       if (firstStep) initializeDatasets(); // this should also create initial random regions  
       if (sequences.isEmpty()) return;
       int selectedSequence=selectNextSequence(selectSequencesAtRandom);
       String selectedSequenceName=sequences.get(selectedSequence).getSequenceName();
       updateMotifFromRegions(selectedSequenceName); // creates a new PWM based on the currently selected regions in all sequences except the one currently selected
       updateBackgroundDistribution(selectedSequenceName); // updates the background distribution based on the DNA base frequencies outside motif regions in all sequences except the one currently selected
       updateProbabilityDistribution(selectedSequenceName); // recalculates the probability distribution in the selected sequence based on scores created with the new motif
       performSamplingStep(selectedSequenceName, numerictrack);  // sample a new region from this sequence    
       updateMotifFromRegions(null); // update the PWM. This time based on ALL the sequences
       stepCounter++;           
    }
    
    /** Create empty data objects to get things started and samples (uniformly) one motif region from each sequence */
    private void initializeDatasets() {
       if (sequences==null || sequences.isEmpty() || dnatrack==null) setupSequences();
       if (sequences==null || sequences.isEmpty() || dnatrack==null) return;
       motifsize=(Integer)motifSizeSpinner.getValue();       
       regiondataset=new RegionDataset(GIBBS_SAMPLING_REGIONS_DATASETNAME);
       regiondataset.setupDefaultDataset(sequences);
       regiondataset.setMotifTrack(true);
       gui.getVisualizationSettings().setVisualizeRegionStrand(regiondataset.getName(), false);
       numerictrack=new NumericDataset(GIBBS_SAMPLING_DISTRIBUTION_DATASETNAME);
       numerictrack.setupDefaultDataset(sequences);  
       currentmotif=new Motif(GIBBS_SAMPLING_MOTIF_DATASETNAME);   
       motiflogopanel.setMotif(currentmotif);
       sampleRegionsRandomly();
       updateMotifFromRegions(null);
    }
    
    private void updateData() {
        try {
           if (regiondataset!=null) engine.updateDataItem(regiondataset);
           if (numerictrack!=null) engine.updateDataItem(numerictrack);
           if (currentmotif!=null) engine.updateDataItem(currentmotif);
       } catch (ExecutionError e) {
           // do nothing
       }       
    }
        
    
    /** Creates one randomly positioned Region in each sequence */
    private void sampleRegionsRandomly() {
       if (sequences==null) return;
       int size=sequences.size();
       for (int i=0;i<size;i++) { // sample position and add one region to each sequence
           Sequence seq=sequences.get(i);
           String sequencename=seq.getSequenceName();           
           RegionSequenceData regionsequence=(RegionSequenceData)regiondataset.getSequenceByName(sequencename);
           if (regionsequence==null) return; // the sequence has disappeard (deleted?)
           regionsequence.clearRegions();
           int sequenceLength=seq.getSize();
           int pos=samplePosition(null, sequenceLength, motifsize);
           int strand=seq.getStrandOrientation(); // only consider one strand (the relative strand)
           Region region=new Region(regionsequence, pos, pos+motifsize-1, "Gibbs_sampling_motif", 100, strand);
           DNASequenceData dnaseq=(DNASequenceData)dnatrack.getSequenceByName(sequencename);
           if (dnaseq==null) return;
           char[] dna=(char[])dnaseq.getValueInInterval(pos, pos+motifsize-1);
           if (strand==Region.REVERSE) dna=MotifLabEngine.reverseSequence(dna);
           region.setSequence(new String(dna));
           regionsequence.addRegion(region);
       }                
    }       

    /**
     * Samples a new region in the selected sequences based on the provided distribution 
     */
    private void performSamplingStep(String sequencename, NumericDataset distributionData) {  
       RegionSequenceData regionsequence=(RegionSequenceData)regiondataset.getSequenceByName(sequencename);
       if (regionsequence==null) return;
       regionsequence.clearRegions();
       NumericSequenceData distribution=(distributionData!=null)?(NumericSequenceData)numerictrack.getSequenceByName(sequencename):null;
       if (distribution==null) return;
       int sequenceLength=regionsequence.getSize();
       int pos=samplePosition(distribution, sequenceLength, motifsize);
       int strand=regionsequence.getStrandOrientation(); // only consider one strand (the relative strand)
       int relativeStart=(strand==Sequence.DIRECT)?pos:pos+motifsize-1;
       double score=distribution.getValueAtRelativePosition(relativeStart);
       Region region=new Region(regionsequence, pos, pos+motifsize-1, "Gibbs_sampling_motif", score, strand);
       DNASequenceData dnaseq=(DNASequenceData)dnatrack.getSequenceByName(sequencename);
       if (dnaseq==null) return;
       char[] dna=(char[])dnaseq.getValueInInterval(pos, pos+motifsize-1);
       if (strand==Region.REVERSE) dna=MotifLabEngine.reverseSequence(dna);
       region.setSequence(new String(dna));
       regionsequence.addRegion(region);                      
    }    
    
    private void updateMotifFromRegions(String holdOutSequenceName) {
       double[][] matrix=getMatrixFromRegions(holdOutSequenceName);
       currentmotif.setMatrix(matrix);          
    }
    
    private double[][] getMatrixFromRegions(String holdOutSequenceName) {
       int size=sequences.size();
       double[][] matrix=new double[motifsize][4];
       for (int i=0;i<size;i++) { // sample position and add one region to each sequence
           Sequence seq=sequences.get(i);
           String sequencename=seq.getSequenceName(); 
           if (holdOutSequenceName!=null && sequencename.equals(holdOutSequenceName)) continue; // do not include this sequence in the PWM
           RegionSequenceData regionsequence=(RegionSequenceData)regiondataset.getSequenceByName(sequencename);
           DNASequenceData dnasequence=(DNASequenceData)dnatrack.getSequenceByName(sequencename);     
           if (regionsequence==null || dnasequence==null) return new double[0][4];
           Region region=regionsequence.getRegionByIndex(0); // each sequence should have exactly one region
           char[] dna=(char[])dnasequence.getValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
           if (region.getOrientation()==Region.REVERSE) dna=MotifLabEngine.reverseSequence(dna);
           for (int j=0;j<dna.length;j++) {
               switch (dna[j]) {
                   case 'A': matrix[j][0]++;break;
                   case 'a': matrix[j][0]++;break;
                   case 'C': matrix[j][1]++;break;
                   case 'c': matrix[j][1]++;break;
                   case 'G': matrix[j][2]++;break;
                   case 'g': matrix[j][2]++;break;
                   case 'T': matrix[j][3]++;break;        
                   case 't': matrix[j][3]++;break;                           
               };
           }                  
       }
       return matrix;        
    }    
    
    private void updateBackgroundDistribution(String holdOutSequenceName) {
       int size=sequences.size();             
       double count=0;
       for (int j=0;j<backgroundDistribution.length;j++) backgroundDistribution[j]=0;
       for (int i=0;i<size;i++) { // sample position and add one region to each sequence
           Sequence seq=sequences.get(i);
           String sequencename=seq.getSequenceName();
           if (sequencename.equals(holdOutSequenceName)) continue; // the sequence hold out should not be included in the background distribution
           boolean reverse=(seq.getStrandOrientation()==Sequence.REVERSE);           
           RegionSequenceData regionsequence=(RegionSequenceData)regiondataset.getSequenceByName(sequencename);
           DNASequenceData dnasequence=(DNASequenceData)dnatrack.getSequenceByName(sequencename);
           if (regionsequence==null || dnasequence==null) return;
           char[] dnaseq=(char[])dnasequence.getValue();
           Region region=regionsequence.getRegionByIndex(0);
           int regionStart=region.getRelativeStart();
           int regionEnd=region.getRelativeEnd();
           for (int j=0;j<dnaseq.length;j++) {
               if (j>=regionStart && j<=regionEnd) continue; // do not count background within the PWM region
               char dna=dnaseq[j];
               switch (dna) {
                   case 'a':case 'A':backgroundDistribution[(reverse)?3:0]++;break;                   
                   case 'c':case 'C':backgroundDistribution[(reverse)?2:1]++;break;                   
                   case 'g':case 'G':backgroundDistribution[(reverse)?1:2]++;break;                   
                   case 't':case 'T':backgroundDistribution[(reverse)?0:3]++;break;                                     
               }
               count++;
           }
       }
       for (int j=0;j<backgroundDistribution.length;j++) backgroundDistribution[j]=backgroundDistribution[j]/count;
    }
    
    /** Selects the next sequence to hold out. Either at random or round-robin. The label color of the chosen sequence will be set to white and the others to black */
    private int selectNextSequence(boolean random) {
        if (sequences.isEmpty()) return 0;
        int selected=0;
        if (random) selected=(int)(Math.random()*sequences.size());
        else selected=stepCounter%sequences.size();
        String selectedSequenceName=sequences.get(selected).getSequenceName();
        for (Sequence seq:sequences) {
            boolean isSelected=seq.getSequenceName().equals(selectedSequenceName);
            Color labelColor=(isSelected)?Color.RED:Color.BLACK;
            gui.getVisualizationSettings().setSequenceLabelColor(seq.getSequenceName(), labelColor);
        }
        return selected;
    }
    
    private void redrawOnEDT() {
        Runnable runner=new Runnable() {
            @Override
            public void run() {
                gui.redraw();
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else try {SwingUtilities.invokeAndWait(runner);} catch(Exception e) {}
    }
    
    /** Adds pseudo counts to a frequency matrix and renormalizes it*/
    private void addPseudo(double[][] matrix, double pseudo) {
        for (int i=0;i<matrix.length;i++) {
            for (int j=0;j<4;j++) {
               matrix[i][j]=(matrix[i][j]+pseudo)/1.0+(pseudo*4);
            }
        }
    }
    
    /** Updates the numeric track containing probability distributions based on the current motif (with length W)
     *  The value Ax at each position X in the sequence should be set to Ax=Qx/Px, where Qx is the probability
     *  that the DNA sequence with length W starting at position X was generated by the motif and Px is the probability
     *  of generating this sequence from the background distribution.
     */
    private void updateProbabilityDistribution(String sequencename) {        
       NumericSequenceData distributionSequence=(NumericSequenceData)numerictrack.getSequenceByName(sequencename);
       DNASequenceData dnaSequence=(DNASequenceData)dnatrack.getSequenceByName(sequencename);
       if (distributionSequence==null || dnaSequence==null) return;
       double[] oldminmax=numerictrack.getMinMaxValuesFromData();
       double[][] matrix=currentmotif.getMatrixAsFrequencyMatrix();
       addPseudo(matrix, 0.03);
       try {
           scoreSequence(dnaSequence, distributionSequence, matrix, dnaSequence.getStrandOrientation());
           //normalizeNumericTrack(distributionSequence); // normalize track sum to 1 for this sequence to make sampling easier
           double[] minmax=distributionSequence.getMinMaxFromData();
           double max=oldminmax[1];
           if (minmax[1]>max) max=minmax[1];           
           numerictrack.setBaselineValue(0);
           numerictrack.setMinAllowedValue(0);
           numerictrack.setMaxAllowedValue(max);               
       } catch (Exception e) {
           engine.logMessage(e.toString());
       }               
    }    
    
    
    /**
     * Returns a new position (and orientation) for where to place the new region
     * in the given sequence
     * @param seqdata A numeric track containing probability distribution (need not be normalized). If NULL then a uniform distribution will be used.
     * @param sequenceLength The length of the sequence
     * @param motiflength The size of the motif
     * @return a the sampled position within the sequence
     */
    private int samplePosition(NumericSequenceData seqdata, int sequenceLength, int motiflength) {
        if (seqdata!=null) {
            return sampleSitePosition(seqdata, motiflength);
        } else { // uniform sampling
            int useLength=sequenceLength-motiflength+1;
            int pos=(int)Math.floor(Math.random()*useLength);
            //int strand=(Math.random()<0.5)?1:-1;
            return pos;
        }
    }
    
    
    
    public void scoreSequence(DNASequenceData sourceSequence, NumericSequenceData targetSequence, double[][] matrix, int strand) throws Exception {      
        int startOffset=sourceSequence.getRegionStart();             
        char[] sequence=(char[])sourceSequence.getValueInGenomicInterval(startOffset, sourceSequence.getRegionEnd());
        int scorestrand=sourceSequence.getStrandOrientation();   
        for (int i=0;i<=sequence.length-motifsize;i++) {
            int pos=i;
            if (scorestrand==Sequence.REVERSE) pos+=(motifsize-1); // note that this is the position that will receive the score. It will correspond to the start or end of the motif depending on the sequence orientation. I.e. the score will always be at the first (most upstream) position of the region
            double[] scores=matrixMatcher(matrix,sequence,i); // match both orientation of the motif to the sequence    
            double comparisonScoreDirect=scores[0];
            double comparisonScoreReverse=scores[1];
            double comparisonScoreDirectBG=scores[2];
            double comparisonScoreReverseBG=scores[3];            
            double score=(scorestrand==Sequence.REVERSE)?comparisonScoreReverse:comparisonScoreDirect; // if only single strand, use either direct or reverse depending on strand choice and sequence orientation
            double scoreBG=(scorestrand==Sequence.REVERSE)?comparisonScoreReverseBG:comparisonScoreDirectBG;
            double newScore=(score/scoreBG); //
            targetSequence.setValueAtRelativePosition(pos,newScore);  //
        }
                      
    }    
        
    /** Returns the probability that the sequence starting at position 'offset' in the sequence was generated by the matrix model
     *  This is done by multiplying the corresponding probabilities (i.e. frequencies) for each position
     *  @return An array with four values:
     *         [0] the probability of generating the sequence according to the matrix (direct strand)
     *         [1] the probability of generating the sequence according to the matrix (reverse strand)
     *         [2] the probability of generating the sequence according to the background model (direct strand)
     *         [3] the probability of generating the sequence according to the background model (reverse strand)
     */
    private double[] matrixMatcher(double[][] matrix, char[] sequence, int offset) {
         double directscore=1.0, reversescore=1.0; // initialize to 1.0 rather than 0 since we are multiplying
         double directscoreBackground=1.0, reversescoreBackground=1.0;
         for (int i=0;i<matrix.length;i++) {
             char base=sequence[i+offset];
             double basefreq=0; 
             double basefreqReverse=0;  
             double basefreqBG=0;
             double basefreqBGReverse=0;                
             switch (base) {
                 case 'A': case 'a': basefreq=matrix[i][0];basefreqReverse=matrix[matrix.length-(i+1)][3];basefreqBG=backgroundDistribution[0];basefreqBGReverse=backgroundDistribution[3];break;
                 case 'C': case 'c': basefreq=matrix[i][1];basefreqReverse=matrix[matrix.length-(i+1)][2];basefreqBG=backgroundDistribution[1];basefreqBGReverse=backgroundDistribution[2];break;
                 case 'G': case 'g': basefreq=matrix[i][2];basefreqReverse=matrix[matrix.length-(i+1)][1];basefreqBG=backgroundDistribution[2];basefreqBGReverse=backgroundDistribution[1];break;
                 case 'T': case 't': basefreq=matrix[i][3];basefreqReverse=matrix[matrix.length-(i+1)][0];basefreqBG=backgroundDistribution[3];basefreqBGReverse=backgroundDistribution[0];break;
             }
             directscore*=basefreq; 
             reversescore*=basefreqReverse;
             directscoreBackground*=basefreqBG; 
             reversescoreBackground*=basefreqBGReverse;
         }       
         return new double[]{directscore,reversescore,directscoreBackground,reversescoreBackground};
    }   
    
  
    
    private void normalizeNumericTrack(NumericSequenceData track) {
        double sum=0;
        for (int pos=0;pos<track.getSize();pos++) {
            sum+=track.getValueAtRelativePosition(pos);
        }    
        if (sum==0) return;
        for (int pos=0;pos<track.getSize();pos++) {
            double value=track.getValueAtRelativePosition(pos);
            double newValue=value/sum;
            track.setValueAtRelativePosition(pos, newValue);
        }           
    }
    
    private int sampleSitePosition(NumericSequenceData prior, int motiflength) {
        double sum=0;
        for (int pos=0;pos<prior.getSize();pos++) {
            sum+=prior.getValueAtRelativePosition(pos);
        }        
        if (sum==0) return 0; // no positive values. We are unable to sample from this distribution.
        double randomnumber=Math.random()*sum;
        if (randomnumber==0) randomnumber=sum; // do not allow value of 0 but allow value of 1 (aka sum);
        double cumsum=0; // cumulative sum. This should sum to 1.0 if the prior has been normalized properly
        boolean direct=!(prior.getStrandOrientation()==Sequence.REVERSE);
        if (direct) {
            for (int pos=0;pos<prior.getSize()-motiflength+1;pos++) {
                cumsum+=prior.getValueAtRelativePosition(pos);
                if (randomnumber<=cumsum) return pos;
            }
        } else {
             for (int pos=prior.getSize()-1;pos>=motiflength-1;pos--) {
                cumsum+=prior.getValueAtRelativePosition(pos);
                if (randomnumber<=cumsum) return pos-motiflength+1;
            }
        }       
        //engine.logMessage("Not able to sample position based prior. I am sorry, this should normally not happen... :|");
        return 0; // not able to find a position
    }    
    
    
    // ----- Plugin interface ------
    
    @Override
    public String getPluginName() {
        return "Gibbs Sampler Demo";
    }

    public String getPluginShortDescription() {
        return "An animated demonstration of the Gibbs Sampler motif discovery method";
    }

    public String getPluginType() {
        return "Tools";
    }

    public String getPluginVersion() {
        return "1.0";
    }

    @Override
    public void initializePlugin(MotifLabEngine engine) throws ExecutionError, SystemError {
        this.engine=engine;
    }

    @Override
    public void initializePluginFromClient(MotifLabClient client) throws ExecutionError, SystemError {
        try {
            if (client instanceof MotifLabGUI) {
                this.gui=(MotifLabGUI)client;  
                ImageIcon bgIcon=new ImageIcon(GibbsSamplerDemo.class.getResource("gibbsicon.png"));
                JMenuItem item=new JMenuItem("Gibbs Sampler Demo");
                item.setIcon(bgIcon); 
                item.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        showDialog();
                    }
                });
                if (!gui.addToToolsMenu(item)) throw new ExecutionError("Unable to add GibbsSamplerDemo to Tools menu");                                                 
            }
        }
        catch (Exception e) {throw new SystemError(e.toString());}
    }
    
    @Override
    public void uninstallPlugin(MotifLabEngine engine) {
       // the dialog is modal so it should not be necessary to close it first
       if (dialog!=null) dialog.setVisible(false);
       if (dialog!=null) dialog.dispose();
       // remove the IdiogramViewer menu item from the Tools menu       
       if (gui instanceof MotifLabGUI) {
           gui.removeFromMenu("Tools", "Gibbs Sampler Demo");           
       }
       gui=null;
       engine=null;
    }

    @Override
    public void shutdownPlugin() { }
    
    
}
